@@ -289,6 +289,8 @@ function (x,
     if (cret$error != empty_string)
         stop(paste(cret$error, "!", sep=""))
 
+    cret$index <- cret$index[1:cret$nr]
+
     ret <- list (
                  call     = match.call(),
                  type     = type,
@@ -309,9 +311,9 @@ function (x,
                  tot.nSV  = cret$nr, #total number of sv
                  nSV      = cret$nSV[1:cret$nclasses], #number of SV in diff. classes
                  labels   = cret$label[1:cret$nclasses], #labels of the SVs.
-                 SV       = if (sparse) SparseM::t(SparseM::t(x[cret$index[1:cret$nr],]))
-                 else t(t(x[cret$index[1:cret$nr],])), #copy of SV
-                 index    = cret$index[1:cret$nr],  #indexes of sv in x
+                 SV       = if (sparse) SparseM::t(SparseM::t(x[cret$index,]))
+                 else t(t(x[cret$index,])), #copy of SV
+                 index    = cret$index,  #indexes of sv in x
                  ##constants in decision functions
                  rho      = cret$rho[1:(cret$nclasses * (cret$nclasses - 1) / 2)],
                  ##probabilites
